/**
 * Account helper
 *
 * @author Pierre HUBERT
 */

#ifndef ACCOUNTHELPER_H
#define ACCOUNTHELPER_H

#include <QObject>
#include <QJsonDocument>

class APIHelper;

#include "../data/accountloginrequest.h"

class AccountHelper : public QObject
{
    Q_OBJECT
public:
    explicit AccountHelper(QObject *parent = nullptr);

    /**
     * Determine wether user is signed in or not
     *
     * @return TRUE if signed in / FALSE else
     */
    bool signedIn();

    /**
     * Logout user
     */
    void logout();

    /**
     * Perform login
     *
     * @param info Request data (user credentials)
     */
    void login(const AccountLoginRequest &info);

signals:

    /**
     * This signal is emitted once a login request has been completed
     *
     * @param result The state of the request
     */
    void loginResult(LoginResult result);

public slots:

private slots:

    //Login request callbacks
    void loginError(int code);
    void requestLoginResult(const QJsonDocument &document);

private:
    APIHelper *mAPIHelper;
};

#endif // ACCOUNTHELPER_H
